// Program chainFilters
/* 
Programme de demonstration sur l'empilage des filtres
Sur une entrée tout ou rien  et sur une entrée analogique
*/
#include <FiltreNum.h>  // Integration de la librairie des filtres numériques 

#define EntreeBool (2)  // on a une entrée tout ou rien (TOR) sur la broche 2 du micro-contrôleur
#define EntreeAna (A0)  // et une entrée analogique en 10 bits (0->1023) sur l'entrée analogique A0 du micro-contrôleur

#define seuil (300)  // le seuil de déclenchement de notre entrée analogique

#define dt (1000)  // dt en microsecondes soit 1ms 0.001s corespond à une fréquence d'acquisition de 1000Hz
// La plage de fréquence d'analyse des filtres est donc au maximum de 1000/2=500Hz (Theoreme de Nyquist)

// Les filtres d'entrées et leur pin d'attachement ou leurs variables de chainages
// déclaration de la broche via une variable
InputType in1 = { in1.pin = EntreeBool };
FilterNumLowPassFirstOrder f1(in1, INPUT_PULLUP, 10);  //un filtre numerique avec une fréquence de coupure de 1/(10*dt*2*PI)=15.9 Hz

FilterTrigger f2(InputType{ .i_ptr = f1.getValIntPtr() }, INPUT_PTR_BOOL, 50);  // un filtre trigger de schmidt chainé au filtre f1 avec un decallage de 50
// ce dernier filtre permet d'éviter les clignotant intempestifs de basculement à un seuil fixe.

// autre manière de déclarer la broche
#define fc (20.)  // Hz
#define n0 int((1000000 / (dt * 2 * M_PI * fc)) + 0.5)
FilterNumLowPassFirstOrder f3(InputType{ .pin = EntreeAna }, INPUT_ANALOG, n0);
FilterNumLowPassFirstOrder f4(InputType{ .i_ptr = f3.getValIntPtr() }, INPUT_PTR_INT, n0);                  // le filtre f3 est chainé au filtre f2
FilterNumLowPassFirstOrder f5(InputType{ .i_ptr = f4.getValIntPtr() }, INPUT_PTR_INT, dt / 1000000., 20.);  // le filtre f4 est chainé au filtre f3
// il est défini par rapport à la fréquence d'acquisition et la fréquence de coupure.
// le résultat en sortie de f3 est un filtre du deuxieme ordre avec un facteur de qualité de 0.5 et une fréquence de coupure fc.
// en sortie de f4 nous avons un filtre du 3ème ordre (-60 db par decade dans la phase de coupure )


void setup() {
  // Creation de la pile chainées de filtres à traiter
  addFilter(f1);
  addFilter(f2);
  addFilter(f3);

  // initialisation de ces filtres
  initFilters();
  // mise en place du timer avec sa valeur de selection
  Timer1.initialize(dt);
  // mise en place de la procédure d'interrruption (effedtuée tout les dt
  // cette fonction s'apelle teatmentFilters dans la bibliothèque Filtre
  Timer1.attachInterrupt(treatmentFilters, 1000);

  // ajouter votre code ici
  Serial.begin(9600);  // par exemple
}
void actionTOR1() {
  // Ce qu'il faut faire si EntreeBool est vrai filtré par f1
}
void actionTOR2() {
  // Ce qu'il faut faire si EntreeBool est vrai filtré par f2(f1)
}
void actionAna1() {
  // Ce qu'il faut faire sir EntreeAna renvoi une valeur > seuil filtré par f3
}
void actionAna2() {
  // Ce qu'il faut faire sir EntreeAna renvoi une valeur > seuil filtré par f4(f3)
}
void loop() {
  if (f1.getValBool()) actionTOR1();
  if (f2.getValBool()) actionTOR2();
  if (f2.getValInt() > seuil) actionAna1();
  if (f3.getValInt() > seuil) actionAna2();
}
