/* 
Program test_all_filters
************************
Je vous livre ici le programme qui m'a permis de debugger la bibliothèque :

C'est un programme de demonstration de traitement de signaux préenregistré
Ce programme fonctionne sur la famille des avr (et certainement d'autres)
Il constitue un bon exemple d'utilisation de la bibliothèque sur des signaux enregistrés
En outre il permet de comprendre via la visualisation du grapheur intégré à l'IDE d'arduino
de visualiser l'effet de filtres sur des signaux.

Les commandes du grapheur sont :
  +     : ajouter une sortie
  -     : enlever une sortie
  *     : signifie toutes les sorties
  1..9  : les sorties de 1 à 9
  A..F  : les sorties de A à F

ex : -*+7EF  enlèvera toutes les sorties puis remettra les sorties 7, E et F
     +* affichera toutes les courbes
Attention :
 Le grapheur est on ne peut plus simple. Les couleurs des courbes sont attachée à leur ordre d'apparition
 I.E. le fait d'ajouter ou d'enlever une courbe provoque le changement de couleur des autres.

A vous de jouer !!
Programme écrit par Yves Cordier le 02/04/2025
debuggé sur processeur Uno, micro, leonaro, mega

*/
// Intégration des bibliotèques
#include <FiltreNum.h>  // Integration de la librairie des filtres numériques
// #include <TimerOne.h> // la bibliothèque du timer1 
// non utile dans ce programme.

// Les signaux à traiter

#ifdef __AVR_ATmega2560__
#define _taille (126) // plus de 650 conduit à une collision entre la pile et le tas => plantage du programme
// on a pris 126=2*pi*20 pour le filtre passe bande
#pragma message("utilisation d'un atmega 2560 taille echantillon 126")
#else // dans le cas d'un uno ...
#define _taille (31) // plus de 50 conduit à une collision entre la pile et le tas => plantage du programme
// on a pris 62=2*pi*5 pour le filtre passe bande
#pragma message("utilisation d'un 386p taille echantillon 31")
#endif

// #define _taille (60)               // plus de 60 conduit à une collision entre la pile et le tas => plantage du programme
int signalCarre[_taille];          // le signal sera stocké sous forme d'un tableau à 60 cases d'entiers => 120 octets
int signalCarrePerturbe[_taille];  // le signal perturbe avec des glyphes
int signalSin[_taille];            // signal sinusioïdale
int signalSinPerturbe[_taille];    // signal perturbé par une harmonique 10
int signalSinComposite[_taille];   // signal sinusoidale + signal carre harmonique 10


// Il faut bien les créer ...

void creeSignaux(int nbPeriodes, int nbperturbations)  // cree les signaux
{
  int tailleMotif = _taille / nbPeriodes;
  for (int i = 0; i < _taille; i++) {
    float arg = (float(i) / tailleMotif * 2 * M_PI);
    while (arg > 2 * M_PI) arg = arg - 2 * M_PI;

    if (arg < M_PI) signalCarre[i] = 1023;
    else signalCarre[i] = 0;
    signalCarrePerturbe[i] = signalCarre[i];
    signalSin[i] = int(1023 * sin(arg) + 0.5);
    signalSinPerturbe[i] = signalSin[i] + int(1023 * sin(10 * arg) + 0.5);
    if (arg < M_PI) signalSinComposite[i] = 1023;
    else signalSinComposite[i] = -1023;
    signalSinComposite[i] += 512 * sin(10 * arg);
  }
  for (int i = 0; i < (_taille / nbperturbations); i++) {
    int j = random(_taille);
    if (signalCarre[j] > 0) signalCarrePerturbe[j] = 0;
    else signalCarrePerturbe[j] = 1023;
  }
}


void initTable() {
  creeSignaux(1, 5); // 2 : nombre de motifs dans le fichier 5 : 5 perturbations par motif
}

// Les variables globales

int s1, s2, s3, s4, s5, sf1, sf2, sf3, sf4, sf5, sf6, sf7, sf8, sf9, sf10;
FilterNumLowPassFirstOrder *f1;
FilterNumLowPassFirstOrder *f2;
FilterNumLowPassFirstOrder *f3;
FilterNumLowPassSecondOrder *f4;
FilterNumHighPassFirstOrder *f5;
FilterNumHighPassSecondOrder *f6;
FilterNumHighPassSecondOrder *f7;
FilterNumPassBandSecondOrder *f8;
FilterNumTrigger *f9;

// récupère les résultats des filtres
void valSignaux() {
  sf1 = f1->getOutVal();
  sf2 = f2->getOutVal();
  sf3 = f3->getOutVal();
  sf4 = f4->getOutVal();
  sf5 = f5->getOutVal();
  sf6 = f6->getOutVal();
  sf7 = f7->getOutVal();
  sf8 = f8->getOutVal();
  sf9 = f9->getOutVal();
  if (f9->getOutBool()) sf10 = 1023;
  else sf10 = 0;
}
int indexTable = 0;

// récupère les valeurs des tableaux
void getValues() {
  s1 = signalCarre[indexTable];          // le signal sera stocké sous forme d'un tableau à 200 cases d'entiers => 400 octets
  s2 = signalCarrePerturbe[indexTable];  // le signal perturbe avec des glyphes
  s3 = signalSin[indexTable];            // signal sinusioïdale
  s4 = signalSinPerturbe[indexTable];    // signal perturbé par une harmonique 10
  s5 = signalSinComposite[indexTable];   // signal carré+ signal sinusoidale harmonique 10
  indexTable++;
  if (indexTable >= _taille) indexTable = 0;
}

// travail répétitif d'affichage des informations sur la voie série
void renseigne() {
  long t1, t2;
  Serial.print("filtre 1 low pass premier ordre fc=");
  Serial.print(f1->getFreqRef());
  Serial.print(" n0 :");
  Serial.print(f1->n);
  Serial.print(", temps execution : ");
  t1 = micros();
  for (int i = 0; i < 100; i++) f1->compute();
  t2 = micros();
  Serial.print((t2 - t1) / 100);
  Serial.println("us");
  Serial.print("filtre 2 low pass premier ordre fc=");
  Serial.print(f2->getFreqRef());
  Serial.print(" n0 :");
  Serial.print(f2->n);
  Serial.print(", temps execution : ");
  t1 = micros();
  for (int i = 0; i < 100; i++) f2->compute();
  t2 = micros();
  Serial.print((t2 - t1) / 100);
  Serial.println("us");
  Serial.print("filtre 3 low pass premier ordre fc=");
  Serial.print(f3->getFreqRef());
  Serial.print(" n0 :");
  Serial.print(f3->n);
  Serial.print(", temps execution : ");
  t1 = micros();
  for (int i = 0; i < 100; i++) f3->compute();
  t2 = micros();
  Serial.print((t2 - t1) / 100);
  Serial.println("us");
  Serial.print("filtre 4 low pass second ordre fc=");
  Serial.print(f4->getFreqRef());
  Serial.print(" Q=");
  Serial.print(f4->getQuality());
  Serial.print(", temps execution : ");
  t1 = micros();
  for (int i = 0; i < 100; i++) f4->compute();
  t2 = micros();
  Serial.print((t2 - t1) / 100);
  Serial.println("us");
  Serial.print("filtre 5 high pass premier ordre fc=");
  Serial.print(f5->getFreqRef());
  Serial.print(" n0 :");
  Serial.print(f5->n);
  Serial.print(", temps execution : ");
  t1 = micros();
  for (int i = 0; i < 100; i++) f5->compute();
  t2 = micros();
  Serial.print((t2 - t1) / 100);
  Serial.println("us");
  Serial.print("filtre 6 high pass second ordre fc=");
  Serial.print(f6->getFreqRef());
  Serial.print(" Q=");
  Serial.print(f6->getQuality());
  Serial.print(", temps execution : ");
  t1 = micros();
  for (int i = 0; i < 100; i++) f6->compute();
  t2 = micros();
  Serial.print((t2 - t1) / 100);
  Serial.println("us");
  Serial.print("filtre 7 high pass second ordre fc=");
  Serial.print(f7->getFreqRef());
  Serial.print(" Q=");
  Serial.print(f7->getQuality());
  Serial.print(", temps execution : ");
  t1 = micros();
  for (int i = 0; i < 100; i++) f7->compute();
  t2 = micros();
  Serial.print((t2 - t1) / 100);
  Serial.println("us");
  Serial.print("filtre 8 band pass second ordre f1=");
  Serial.print(f8->getFreqf1());
  Serial.print(" f2=");
  Serial.print(f8->getFreqf2());
  Serial.print(", temps execution : ");
  t1 = micros();
  for (int i = 0; i < 100; i++) f8->compute();
  t2 = micros();
  Serial.print((t2 - t1) / 100);
  Serial.println("us");
  Serial.print("filtre 9 trigger de schmidt ");
  Serial.print(", temps execution : ");
  t1 = micros();
  for (int i = 0; i < 100; i++) f9->compute();
  t2 = micros();
  Serial.print((t2 - t1) / 100);
  Serial.println("us");
}
unsigned int visu = 0xffff;

// travail répétitif d'affichage
void afficheCourbes() {
  if (visu & 0x0001) // signal N°1
  {
    Serial.print("cp:");Serial.print(s1);  // creneau positif
    Serial.print('\t');
  }
  if (visu & 0x0002) // signal N°2
  {
    Serial.print("cpP:");Serial.print(s2);  // creneau perturbe
    Serial.print('\t');
  }
  if (visu & 0x0004ul) // signal N°3
  {
    Serial.print("sin:");Serial.print(s3);  // sinus pur
    Serial.print('\t');
  }
  if (visu & 0x0008) // signal N°4
  {
    Serial.print("sinP:");Serial.print(s4);  // sinus perturbe
    Serial.print('\t');
  }
  if (visu & 0x0010) // signal N°5
  {
    Serial.print("crP:");Serial.print(s5);  // creneau symetrique perturbe par sin
    Serial.print('\t');
  }
  if (visu & 0x0020) // signal N°6
  {
    Serial.print(sf1);  // creneau positif filtre passe bas 1er ordre
    Serial.print('\t');
  }
  if (visu & 0x0040) // signal N°7
  {
    Serial.print("f1(cpP):");Serial.print(sf2);  // creneau positif perturbe filtre passe bas 1er ordre
    Serial.print('\t');
  }
  if (visu & 0x0080) // signal N°8
  {
    Serial.print("f1(f1(cpP)):");Serial.print(sf3);  // sf1 filtré passe bas 1er ordre => creneau positif filtré 2eme ordre Q=0.5
    Serial.print('\t');
  }
  if (visu & 0x0100) // signal N°9
  {
    Serial.print("f2(cpP):");Serial.print(sf4);  // creneau positif filtre passe bas 2eme ordre Q=0.5
    Serial.print('\t');
  }
  if (visu & 0x0200) // signal N°A
  {
    Serial.print("ph1(cp):");Serial.print(sf5);  // creneau positif filtre passe haut 1er ordre
    Serial.print('\t');
  }
  if (visu & 0x0400) // signal N°B
  {
    Serial.print("ph2(cp):");Serial.print(sf6);  // creneau positif filtre passe haut 2eme ordre
    Serial.print('\t');
  }
  if (visu & 0x0800) // signal N°C
  {
    Serial.print("ph2(cpP):");Serial.print(sf7);  // creneau positif perturbe  filtre passe haut 2eme ordre
    Serial.print('\t');
  }
  if (visu & 0x1000) // signal N°D
  {
    Serial.print("pb2(crP):");Serial.print(sf8);  // signal composite filtre pass bande 2eme ordre
    Serial.print('\t');
  }
  if (visu & 0x2000) // signal N°E
  {Serial.print("trA(f1(cpP)):");Serial.print(sf9);  // sortie "analogique" du trigger de shmidt
  Serial.print('\t');
  }
  if (visu & 0x4000) // signal N°F
  {Serial.print("trB(f1(cpP):");Serial.print(sf10); } // Sortie booleenne du trigger de schmidt
  Serial.println('\t');
}

// les initialisation
// C'est ici que vous changerez les valeurs caractéristiques de vos filtres.
void setup() {
  Serial.begin(57600);
  delay(1000);
  initTable();
  f1 = new FilterNumLowPassFirstOrder(InputType{ .i_ptr = &s1 }, INPUT_PTR_INT, 4);
  f2 = new FilterNumLowPassFirstOrder(InputType{ .i_ptr = &s2 }, INPUT_PTR_INT, 4);
  f3 = new FilterNumLowPassFirstOrder(InputType{ .i_ptr = f1->getValOutPtr() }, INPUT_PTR_INT, 4);
  f4 = new FilterNumLowPassSecondOrder(InputType{ .i_ptr = &s1 }, INPUT_PTR_INT, 8, 4);  // même tau que f1+f3 et Q=0.5
  f5 = new FilterNumHighPassFirstOrder(InputType{ .i_ptr = &s1 }, INPUT_PTR_INT, 4);
  f6 = new FilterNumHighPassSecondOrder(InputType{ .i_ptr = &s1 }, INPUT_PTR_INT, 8, 4);  // même tau que f1+f3 et Q=0.5
  f7 = new FilterNumHighPassSecondOrder(InputType{ .i_ptr = &s2 }, INPUT_PTR_INT, 8, 20);
  f8 = new FilterNumPassBandSecondOrder(InputType{ .i_ptr = &s5 }, INPUT_PTR_INT, 1., 1. / _taille*10, 3); // freq de la perturbation
  f9 = new FilterNumTrigger(InputType{ .i_ptr = f2->getValOutPtr() }, INPUT_PTR_INT, 400,600,0,1023);  // trigger chainé à la sortie filtré du signal creneau bruité
}
char phrase[10] = { 0 };

void analysePhrase() {
  bool sens = true;
  byte index = 0;
  while ((index <= (sizeof(phrase) - 1)) && (phrase[index] != 0)) {
    char ch = phrase[index];
    if (ch == '+') sens = true;
    if (ch == '-') sens = false;
    if ((ch == '0') || (ch == '*')) {
      if (sens) visu = 0xFFFF;
      else visu = 0x0000;
    }
    if ((ch >= '1') && (ch <= '9')) {
      if (sens) visu |= (0x0001 << (ch - '1'));
      else visu &= ~(0x0001 << (ch - '1'));
    }
    if ((ch >= 'A') && (ch <= 'L')) {
      if (sens) visu |= (0x0200 << (ch - 'A'));
      else visu &= ~(0x0200 << (ch - 'A'));
    }
    index++;
  }
  phrase[0] = 0;
  delay(100); // pour satisfaire le grapheur
}
char upper(char c) {
  if ((c >= 'a') && (c <= 'z')) c += 'A' - 'a';
  return c;
}

void ajoute(char c, char p[]) {
  byte i = 0;
  while ((i < (sizeof(phrase) - 1)) && (p[i] != 0)) i++;
  p[i] = c;
  p[i + 1] = 0;
}


void command() {
  if (Serial.available()) {
    char ch = Serial.read();
    if ((ch == 10) || (ch == 13)) {
      analysePhrase();
      Serial.println();
      return;
    }
    if ((ch == '+') || (ch == '-') || (ch == '*')) {
      ajoute(ch, phrase);
      return;
    }
    ch = upper(ch);

    if (((ch >= '0') && (ch <= '9')) || ((ch >= 'A') && (ch <= 'L'))) {
      ajoute(ch, phrase);
      return;
    }
  }
}


byte etape = 0;
int compte = 0;

// L'automate d'état fini de l'application
void loop() {
  switch (etape) {
    case 0:
      renseigne();  // affiche un point textuel sur les entrées sorties
                    // ajout des filtres dans la pile de filtres
      addFilter(f1);
      addFilter(f2);
      addFilter(f3);
      addFilter(f4);
      addFilter(f5);
      addFilter(f6);
      addFilter(f7);
      addFilter(f8);
      addFilter(f9);
      initFilters();
      etape++;
      break;
    case 1:  // on attends l'action clavier
      Serial.flush();
      Serial.println("appuyez une touche pour continuer (penser à basculer en mode traceur)");
      Serial.println("Les commandes sont [[+-][*,0-9][a-l][A-L]]+  pour afficher/masquer une ou plusieurs courbes");
      etape++;
      compte = 0;
      break;
    case 2:
      if (Serial.available()) {
        etape++;
        break;
      }
      compte++;
      if (compte > 500) {
        etape = 1;
        break;
      }
      delay(10);
      break;
    case 3:  // on avance d'un pas sur le tableau des signaux à chaque tour
      command();
      getValues();       // actualise les valeurs d'entrées
      interTimer();      // utilisé seul comme si dt=1
      valSignaux();      // actualise les valeurs de sortie
      afficheCourbes();  // c'est écrit dans le titre !
      break;
  }
}
